
/*!
* Module dependencies
*/

var mongoose = require('mongoose');
var crypto = require('crypto');

var Schema = mongoose.Schema;
const oAuthTypes = [
  'github',
  'twitter',
  'facebook',
  'google',
  'linkedin'
];


/**
* User schema
*/

var UserSchema = new Schema({
    name: {
        type: String,
        required: true
    },
    email: {
        type: String,
        required: true,
        unique: true
    },
    hashed_password: {
        type: String,
        default: ''
    },
    provider: {
        type: String,
        default: ''
    },
    google: {},
    salt: {
        type: String,
        default: ''
    }
});

/**
* Virtuals
*/

UserSchema
.virtual('password')
.set(function(password) {
    this._password = password;
    this.salt = this.makeSalt();
    this.hashed_password = this.encryptPassword(password);
})
.get(function() {
    return this._password;
})

/**
* Validations
*/

var validatePresenceOf = function (value) {
    return value && value.length;
};

// the below 5 validations only apply if you are signing up traditionally

UserSchema.path('name').validate(function (firstName) {
    if (this.skipValidation()) return true;
    return name.length;
}, 'Name cannot be blank');

UserSchema.path('email').validate(function (email) {
    if (this.skipValidation()) return true;
    return email.length;
}, 'Email cannot be blank');

UserSchema.path('email').validate(function (email, fn) {
    var User = mongoose.model('User');
    if (this.skipValidation()) fn(true);

    // Check only when it is a new user or when email field is modified
    if (this.isNew || this.isModified('email')) {
        User.find({ email: email }).exec(function (err, users) {
            fn(!err && users.length === 0);
        });
    } else fn(true);
}, 'Email already exists');

UserSchema.path('hashed_password').validate(function (hashed_password) {
    if (this.skipValidation()) return true;
    if(this.isNew || this._password) {
        return hashed_password.length && this._password.length;
    } else {
        return true;
    }
}, 'Password cannot be blank');



/**
* Add your
* - pre-save hooks
* - validations
* - virtuals
*/

UserSchema.pre('save', function(next) {
    if (!this.isNew) return next();

    if (!validatePresenceOf(this.password) && !this.skipValidation()) {
        next(new Error('Invalid password'));
    } else {
        next();
    }
})

/**
* Methods
*/

UserSchema.method({
    /**
    * Authenticate - check if the passwords are the same
    *
    * @param {String} plainText
    * @return {Boolean}
    * @api public
    */

    authenticate: function (plainText) {
        return this.encryptPassword(plainText) === this.hashed_password;
    },

    /**
    * Make salt
    *
    * @return {String}
    * @api public
    */

    makeSalt: function () {
        return Math.round((new Date().valueOf() * Math.random())) + '';
    },

    /**
    * Encrypt password
    *
    * @param {String} password
    * @return {String}
    * @api public
    */

    encryptPassword: function (password) {
        if (!password) return '';
        try {
            return crypto
            .createHmac('sha1', this.salt)
            .update(password)
            .digest('hex');
        } catch (err) {
            console.log('err', err);
            return '';
        }
    },

    /**
    * Validation is not required if using OAuth
    */

    skipValidation: function() {
        return ~oAuthTypes.indexOf(this.provider);
    }
});

/**
* Statics
*/

UserSchema.static({
    /**
    * Load
    *
    * @param {Object} options
    * @param {Function} cb
    * @api private
    */

    load: function (options, cb) {
        options.select = options.select || 'name';
        this.findOne(options.criteria)
        .select(options.select)
        .exec(cb);
    }
});

/**
* Register
*/

mongoose.model('User', UserSchema);
